package net.sf.twip.mockito;

import net.sf.twip.TwipExtension;

import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.Statement;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;

/**
 * Allows you to use the {@link org.mockito.Mockito mockito} {@link Mock @Mock}
 * annotation.
 */
public class MockitoExtension implements TwipExtension {

	private static final class MockitoInitStatement extends Statement {
		private final Statement statement;
		private final Object testInstance;

		public MockitoInitStatement(Statement statement, Object testInstance) {
			this.statement = statement;
			this.testInstance = testInstance;
		}

		@Override
		public void evaluate() throws Throwable {
			MockitoAnnotations.initMocks(testInstance);
			statement.evaluate();
		}
	}

	private static final class MockitoValidateStatement extends Statement {
		private final Statement statement;

		public MockitoValidateStatement(Statement statement) {
			this.statement = statement;
		}

		@Override
		public void evaluate() throws Throwable {
			statement.evaluate();
			Mockito.validateMockitoUsage();
		}
	}

	public Statement wrap(boolean beforeSetup, FrameworkMethod method, Object testInstance,
			Statement statement) {
		return beforeSetup ? new MockitoValidateStatement(statement) : new MockitoInitStatement(statement,
				testInstance);
	}
}
